
/*
 bench.c : a simple benchmark for 68k emulations
*/

#include "prz.h"

void check_time(void)
{
  long t0,t1;
  int i;
  printf("checking time. Measuring 5 seconds\n");
  printf("(use your stopwatch between the two '0')\n");
  for(i = 3 ; i >= 0 ; i--) {
    printf("%d ", i);
    t0 = clock();
    do {
      t1 = clock();
    } while(t1 < t0 + CLOCKS_PER_SEC);
  }
  printf("\n");
  for(i = 5 ; i >= 1 ; i--) {
    printf("%d ", i);
    t0 = clock();
    do {
      t1 = clock();
    } while(t1 < t0 + CLOCKS_PER_SEC);
  }
  printf("0 done\n");
}

void time(void (*f)(void), int rpt, char *fname)
{
  long t0, t1;
  long n;
  int i;
  
  t0 = clock();
  printf("time for 10 million %s (in ms): ", fname);
  t0 = clock();
  i = 0;
  do {
    i++;
    f();
    t1 = clock();
  } while(t1 < t0 + 5*CLOCKS_PER_SEC);
  n =  ((t1-t0)*10000)/(i*rpt*CLOCKS_PER_SEC);
  printf("%ld\n", n);
}

extern void bench_nop(void);
extern void bench_moveq(void);
extern void bench_regs(void);
extern void bench_lregs(void);
extern void bench_abs(void);
extern void bench_movem(void);
extern void bench_stack(void);
extern void bench_bsr(void);

int main(int argc, char *argv[])
{
  printf("Hello\n");
  check_time();

  /* 1000000 nops */
  time(bench_nop,   10, "nop           ");
  /* 1000000 moveq */
  time(bench_moveq, 10, "moveq         ");
  /* 1000000  */
  time(bench_regs,  10, "move d1,d2    ");
  /* 1000000  */
  time(bench_lregs, 10, "add.l d1,d2   ");
  /* 1000000  */
  time(bench_abs,   10, "move.l x,x    ");
  /* 100000 */
  time(bench_movem,  1, "movem 48 bytes");
  /* 1000000 */
  time(bench_stack, 10, "pea/pop long  ");
  /* 500000 */
  time(bench_bsr,    5, "bsr/rts       ");

  printf("Done\n");
}
