/*
  longlong.c : dummy calculations involving long long ints, 
*/

#ifndef _ULONG_TYPE
#define _ULONG_TYPE
typedef unsigned long ulong;
#endif



struct ll {
  ulong h;
  ulong l;
};

typedef struct ll *LL;

#ifdef LLTEST
#include <stdio.h>
#define t(m,d) {(void)printf(#m " ");trace(#d,d);}
#define tt(m,d) {(void)printf(#m " %ld\n",(long)d);}
void trace(char *name, LL d)
{
  printf("%s->h=%ld, %s->l=%ld\n", name, d->h, name, d->l);
}
#else
#endif

/* b = b + a */
void ll_add(LL a, LL b) {
  b->l += a->l;
  if(b->l <= a->l) {
    /* report carry */
    b->h ++;
  }
  b->h += a->h;
}

/* b = b - a */
void ll_sub(LL a, LL b) {
  ulong t = b->l;
  b->l -= a->l;
  if(b->l >= t) {
    /* report carry */
    b->h --;
  }
  b->h -= a->h;
}

/* a <=> b , returns -1, 0 or 1.*/
int ll_cmp(LL a, LL b) {
  if (a->h < b->h) {
    return -1;
  } else if(a->h > b->h) {
    return 1;
  } else if(a->l < b->l) {
    return -1;
  } else if(a->l > b->l) {
    return 1;
  } else {
    return 0;
  }
}

void ll_from_long(ulong a, LL b)
{
  b->l = a;
  b->h = 0;
}

void ll_lsl(LL a, unsigned int shift)
{
  ulong t;
  if(shift == 0) return;
  t = a->l >> (32 - shift) ;
  a->l <<= shift;
  a->h <<= shift;
  a->h |= t;
}

/* c = a * b */
void ll_mul(ulong a, ulong b, LL c)
{
  ulong al, ah, bl, bh;
  struct ll dd;
  LL d = &dd;
  
  al = a & 0xFFFF;
  ah = a>>16;
  bl = b & 0xFFFF;
  bh = b>>16;
  c->l = al * bl;
  c->h = ah * bh;
  ll_from_long(al * bh, d);
  ll_lsl(d, 16);
  ll_add(d, c);
  ll_from_long(ah * bl, d);
  ll_lsl(d, 16);
  ll_add(d, c);
}

/* returns b / a, or -1 if overflow. */
long ll_div(LL a, LL b)
{
  int i, r;
  struct ll dd, ee;
  LL d = &dd, e = &ee;
  long c;
  
  e->l = b->l ; e->h = b->h;
  c = 0;
  r = ll_cmp(a, b);
  if(r >= 0) {
    if(r == 0) return 1;
    else return 0;
  } 

  for(i = 30 ; i >= 0 ; i--) {
    d->l = a->l; d->h = a->h;
    /* if e >= (a << i), then { c += (1 << i) ; e -= (a<<i) } */
    ll_lsl(d,i);
    r = ll_cmp(d, e);
    if(r <= 0) {
      ll_sub(d, e);
      c += (1<<i);
      if(r == 0) return c;
    }
  }
  if(ll_cmp(e,a) > 0) {
    /* if remainder > divisor */
    return -1;
  }
  return c;
}

/* returns a * b / c */
long muldiv(long a, long b, long c)
{
  struct ll dd, ee;
  LL d = &dd, e = &ee;
  ll_mul(a, b, d);
  ll_from_long(c, e);
  return ll_div(e, d);
}

#ifdef LLTEST
#include <stdio.h>
#include <stdlib.h>

int main(int argc, char *argv[])
{
  long a, b, c, d;
  
  (void)argc;
  (void)argv;
  
  do {
    printf("a b c = ");
    fflush(stdout);
    scanf("%ld %ld %ld", &a, &b, &c);
    printf("a b c = %ld %ld %ld ", a, b, c);
    d = muldiv(a,b,c);
    printf(">> d = %ld\n", d);
  }
  while(c != 0);
  exit(0);
}
  
#endif
