
/*
  some stdlib functions for PRZ glue
*/

#include "prz.h"

/****************************** malloc **************************/

#define MEM_SIZE 10000

char mem_pool[MEM_SIZE];
long mem_left = 0;

void *lmalloc(long size)
{
  char *result;
  if(mem_left + size >= MEM_SIZE) return 0;
  result = mem_pool+mem_left;
  mem_left += size;
  return (void *)result;
}

void *malloc(int size)
{
  return lmalloc((long)size);
}

void free(void *ptr)
{
  (void) ptr;
}

/****************************** strcmp **************************/

/* strcmp - compare string s1 to s2 */

/* -1 for <, 0 for ==, 1 for > */
int strcmp(char *s1, char *s2)			
{
  register char *scan1;
  register char *scan2;

  scan1 = s1;
  scan2 = s2;
  while (*scan1 != '\0' && *scan1 == *scan2) {
    scan1++;
    scan2++;
  }

  /* The following case analysis is necessary so that characters which
   * look negative collate low against normal characters but high
   * against the end-of-string NUL. */
  if (*scan1 == '\0' && *scan2 == '\0') {
    return(0);
  } else if (*scan1 == '\0') {
    return(-1);
  } else if (*scan2 == '\0') {
    return(1);
  } else {
    return(*scan1 - *scan2);
  }
}

/****************************** strcpy **************************/

/* strcpy - copy string src to dst  */
char *strcpy(char *dst, char *src);

char *strcpy(char *dst, char *src)
{
  register char *dscan;
  register char *sscan;

  dscan = dst;
  sscan = src;
  while ((*dscan++ = *sscan++) != '\0') continue;
  return(dst);
}

/****************************** strlen **************************/

int strlen(char *a)
{
  int len = 0;
  while(*a++) {
    len++;
    if(len < 0) {
      printf("assert failed in strlen. Aborting");
      exit(1);
    }
  }
  return len;
}

/****************************** memcpy **************************/
/* memcpy - copy bytes */

void *lmemcpy(void *dst, void *src, long size);

void *memcpy(void *dst, void *src, int size)
{
  return lmemcpy(dst, src, (long) size);
}

void *lmemcpy(void *dst, void *src, long size)
{
  register char *d;
  register char *s;
  register long n;

  if (size <= 0) return(dst);

  s = (char *) src;
  d = (char *) dst;
  if (s <= d && s + (size - 1) >= d) {
	/* Overlap, must copy right-to-left. */
	s += size - 1;
	d += size - 1;
	for (n = size; n > 0; n--) *d-- = *s--;
  } else
	for (n = size; n > 0; n--) *d++ = *s++;

  return(dst);
}
